# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_utils import versionutils

from senlin.common import consts
from senlin.objects import base
from senlin.objects import fields


@base.SenlinObjectRegistry.register
class ReceiverCreateRequestBody(base.SenlinObject):

    fields = {
        'name': fields.NameField(),
        'type': fields.ReceiverTypeField(),
        'cluster_id': fields.StringField(nullable=True),
        'action': fields.ClusterActionNameField(nullable=True),
        'actor': fields.JsonField(nullable=True, default={}),
        'params': fields.JsonField(nullable=True, default={})
    }


@base.SenlinObjectRegistry.register
class ReceiverCreateRequest(base.SenlinObject):

    fields = {
        'receiver': fields.ObjectField('ReceiverCreateRequestBody')
    }


@base.SenlinObjectRegistry.register
class ReceiverListRequest(base.SenlinObject):

    # VERSION 1.0: Initial version
    # VERSION 1.1: Add field 'user'
    VERSION = '1.1'
    VERSION_MAP = {
        '1.4': '1.1',
    }

    fields = {
        'name': fields.ListOfStringsField(nullable=True),
        'type': fields.ListOfEnumField(
            valid_values=list(consts.RECEIVER_TYPES), nullable=True),
        'action': fields.ListOfEnumField(
            valid_values=list(consts.CLUSTER_ACTION_NAMES), nullable=True),
        'cluster_id': fields.ListOfStringsField(nullable=True),
        'limit': fields.NonNegativeIntegerField(nullable=True),
        'marker': fields.UUIDField(nullable=True),
        'sort': fields.SortField(
            valid_keys=list(consts.RECEIVER_SORT_KEYS), nullable=True),
        'project_safe': fields.FlexibleBooleanField(default=True),
        'user': fields.ListOfStringsField(nullable=True),
    }

    def obj_make_compatible(self, primitive, target_version):
        super(ReceiverListRequest, self).obj_make_compatible(
            primitive, target_version)
        target_version = versionutils.convert_version_to_tuple(
            target_version)
        if target_version < (1, 1):
            if 'user' in primitive:
                del primitive['user']


@base.SenlinObjectRegistry.register
class ReceiverGetRequest(base.SenlinObject):

    fields = {
        'identity': fields.StringField()
    }


@base.SenlinObjectRegistry.register
class ReceiverDeleteRequest(base.SenlinObject):

    fields = {
        'identity': fields.StringField()
    }


@base.SenlinObjectRegistry.register
class ReceiverNotifyRequest(base.SenlinObject):

    fields = {
        'identity': fields.StringField()
    }



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Policy for deleting node(s) from a cluster.

NOTE: For full documentation about how the deletion policy works, check:
http://docs.openstack.org/developer/senlin/developer/policies/deletion_v1.html
"""
from oslo_log import log as logging

from senlin.common import constraints
from senlin.common import consts
from senlin.common.i18n import _
from senlin.common import scaleutils
from senlin.common import schema
from senlin.engine import cluster as cm
from senlin.objects import cluster as co
from senlin.objects import node as no
from senlin.policies import base

LOG = logging.getLogger(__name__)


class DeletionPolicy(base.Policy):
    """Policy for choosing victim node(s) from a cluster for deletion.

    This policy is enforced when nodes are to be removed from a cluster.
    It will yield an ordered list of candidates for deletion based on user
    specified criteria.
    """

    VERSION = '1.0'
    VERSIONS = {
        '1.0': [
            {'status': consts.SUPPORTED, 'since': '2016.04'}
        ]
    }
    PRIORITY = 400

    KEYS = (
        CRITERIA, DESTROY_AFTER_DELETION, GRACE_PERIOD,
        REDUCE_DESIRED_CAPACITY,
    ) = (
        'criteria', 'destroy_after_deletion', 'grace_period',
        'reduce_desired_capacity',
    )

    CRITERIA_VALUES = (
        OLDEST_FIRST, OLDEST_PROFILE_FIRST, YOUNGEST_FIRST, RANDOM,
    ) = (
        'OLDEST_FIRST', 'OLDEST_PROFILE_FIRST', 'YOUNGEST_FIRST', 'RANDOM',
    )

    TARGET = [
        ('BEFORE', consts.CLUSTER_SCALE_IN),
        ('BEFORE', consts.CLUSTER_DEL_NODES),
        ('BEFORE', consts.CLUSTER_RESIZE),
        ('BEFORE', consts.NODE_DELETE),
    ]

    PROFILE_TYPE = [
        'ANY'
    ]

    properties_schema = {
        CRITERIA: schema.String(
            _('Criteria used in selecting candidates for deletion'),
            default=RANDOM,
            constraints=[
                constraints.AllowedValues(CRITERIA_VALUES),
            ]
        ),
        DESTROY_AFTER_DELETION: schema.Boolean(
            _('Whether a node should be completely destroyed after '
              'deletion. Default to True'),
            default=True,
        ),
        GRACE_PERIOD: schema.Integer(
            _('Number of seconds before real deletion happens.'),
            default=0,
        ),
        REDUCE_DESIRED_CAPACITY: schema.Boolean(
            _('Whether the desired capacity of the cluster should be '
              'reduced along the deletion. Default to False.'),
            default=True,
        )
    }

    def __init__(self, name, spec, **kwargs):
        super(DeletionPolicy, self).__init__(name, spec, **kwargs)

        self.criteria = self.properties[self.CRITERIA]
        self.grace_period = self.properties[self.GRACE_PERIOD]
        self.destroy_after_deletion = self.properties[
            self.DESTROY_AFTER_DELETION]
        self.reduce_desired_capacity = self.properties[
            self.REDUCE_DESIRED_CAPACITY]

    def _victims_by_regions(self, cluster, regions):
        victims = []
        for region in sorted(regions.keys()):
            count = regions[region]
            nodes = cluster.nodes_by_region(region)
            if self.criteria == self.RANDOM:
                candidates = scaleutils.nodes_by_random(nodes, count)
            elif self.criteria == self.OLDEST_PROFILE_FIRST:
                candidates = scaleutils.nodes_by_profile_age(nodes, count)
            elif self.criteria == self.OLDEST_FIRST:
                candidates = scaleutils.nodes_by_age(nodes, count, True)
            else:
                candidates = scaleutils.nodes_by_age(nodes, count, False)

            victims.extend(candidates)

        return victims

    def _victims_by_zones(self, cluster, zones):
        victims = []
        for zone in sorted(zones.keys()):
            count = zones[zone]
            nodes = cluster.nodes_by_zone(zone)
            if self.criteria == self.RANDOM:
                candidates = scaleutils.nodes_by_random(nodes, count)
            elif self.criteria == self.OLDEST_PROFILE_FIRST:
                candidates = scaleutils.nodes_by_profile_age(nodes, count)
            elif self.criteria == self.OLDEST_FIRST:
                candidates = scaleutils.nodes_by_age(nodes, count, True)
            else:
                candidates = scaleutils.nodes_by_age(nodes, count, False)

            victims.extend(candidates)

        return victims

    def _update_action(self, action, victims):
        pd = action.data.get('deletion', {})
        pd['count'] = len(victims)
        pd['candidates'] = victims
        pd['destroy_after_deletion'] = self.destroy_after_deletion
        pd['grace_period'] = self.grace_period
        pd['reduce_desired_capacity'] = self.reduce_desired_capacity
        action.data.update({
            'status': base.CHECK_OK,
            'reason': _('Candidates generated'),
            'deletion': pd
        })
        action.store(action.context)

    def pre_op(self, cluster_id, action):
        """Choose victims that can be deleted.

        :param cluster_id: ID of the cluster to be handled.
        :param action: The action object that triggered this policy.
        """

        victims = action.inputs.get('candidates', [])
        if len(victims) > 0:
            self._update_action(action, victims)
            return

        if action.action == consts.NODE_DELETE:
            self._update_action(action, [action.node.id])
            return

        db_cluster = None
        regions = None
        zones = None

        deletion = action.data.get('deletion', {})
        if deletion:
            # there are policy decisions
            count = deletion['count']
            regions = deletion.get('regions', None)
            zones = deletion.get('zones', None)
        # No policy decision, check action itself: SCALE_IN
        elif action.action == consts.CLUSTER_SCALE_IN:
            count = action.inputs.get('count', 1)

        # No policy decision, check action itself: RESIZE
        else:
            db_cluster = co.Cluster.get(action.context, cluster_id)
            current = no.Node.count_by_cluster(action.context, cluster_id)
            res, reason = scaleutils.parse_resize_params(action, db_cluster,
                                                         current)
            if res == base.CHECK_ERROR:
                action.data['status'] = base.CHECK_ERROR
                action.data['reason'] = reason
                LOG.error(reason)
                return

            if 'deletion' not in action.data:
                return
            count = action.data['deletion']['count']

        cluster = cm.Cluster.load(action.context, dbcluster=db_cluster,
                                  cluster_id=cluster_id)
        # Cross-region
        if regions:
            victims = self._victims_by_regions(cluster, regions)
            self._update_action(action, victims)
            return

        # Cross-AZ
        if zones:
            victims = self._victims_by_zones(cluster, zones)
            self._update_action(action, victims)
            return

        if count > len(cluster.nodes):
            count = len(cluster.nodes)

        if self.criteria == self.RANDOM:
            victims = scaleutils.nodes_by_random(cluster.nodes, count)
        elif self.criteria == self.OLDEST_PROFILE_FIRST:
            victims = scaleutils.nodes_by_profile_age(cluster.nodes, count)
        elif self.criteria == self.OLDEST_FIRST:
            victims = scaleutils.nodes_by_age(cluster.nodes, count, True)
        else:
            victims = scaleutils.nodes_by_age(cluster.nodes, count, False)

        self._update_action(action, victims)
        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Webhook endpoint for Senlin v1 ReST API.
"""

from webob import exc

from senlin.api.common import util
from senlin.api.common import wsgi
from senlin.common import consts
from senlin.common.i18n import _


class ReceiverController(wsgi.Controller):
    """WSGI controller for receiver resource in Senlin v1 API."""

    REQUEST_SCOPE = 'receivers'

    @util.policy_enforce
    def index(self, req):
        whitelist = {
            consts.RECEIVER_NAME: 'mixed',
            consts.RECEIVER_TYPE: 'mixed',
            consts.RECEIVER_CLUSTER_ID: 'mixed',
            consts.RECEIVER_USER_ID: 'mixed',
            consts.RECEIVER_ACTION: 'mixed',
            consts.PARAM_LIMIT: 'single',
            consts.PARAM_MARKER: 'single',
            consts.PARAM_SORT: 'single',
            consts.PARAM_GLOBAL_PROJECT: 'single',
        }
        for key in req.params.keys():
            if key not in whitelist.keys():
                raise exc.HTTPBadRequest(_('Invalid parameter %s') % key)

        params = util.get_allowed_params(req.params, whitelist)

        project_safe = not util.parse_bool_param(
            consts.PARAM_GLOBAL_PROJECT,
            params.pop(consts.PARAM_GLOBAL_PROJECT, False))
        params['project_safe'] = project_safe

        obj = util.parse_request('ReceiverListRequest', req, params)
        receivers = self.rpc_client.call(req.context, 'receiver_list', obj)

        return {'receivers': receivers}

    @util.policy_enforce
    def create(self, req, body):

        obj = util.parse_request(
            'ReceiverCreateRequest', req, body, 'receiver')
        result = self.rpc_client.call(req.context, 'receiver_create',
                                      obj.receiver)

        return {'receiver': result}

    @util.policy_enforce
    def get(self, req, receiver_id):
        obj = util.parse_request(
            'ReceiverGetRequest', req, {'identity': receiver_id})
        receiver = self.rpc_client.call(req.context, 'receiver_get', obj)
        return {'receiver': receiver}

    @util.policy_enforce
    def delete(self, req, receiver_id):

        obj = util.parse_request(
            'ReceiverDeleteRequest', req, {'identity': receiver_id})
        self.rpc_client.call(req.context, 'receiver_delete', obj)
        raise exc.HTTPNoContent()

    @util.policy_enforce
    def notify(self, req, receiver_id, body=None):

        obj = util.parse_request(
            'ReceiverNotifyRequest', req, {'identity': receiver_id})
        self.rpc_client.call(req.context, 'receiver_notify', obj)
        raise exc.HTTPNoContent()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestNodeList(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestNodeList, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        self.node_id = utils.create_a_node(self, profile_id)
        self.addCleanup(utils.delete_a_node, self, self.node_id)

    @decorators.idempotent_id('cd086dcb-7509-4125-adfc-6beb63b10d0a')
    def test_node_list(self):
        res = self.client.list_objs('nodes')

        # Verify resp of node list API
        self.assertEqual(200, res['status'])
        self.assertIsNone(res['location'])
        self.assertIsNotNone(res['body'])
        nodes = res['body']
        node_ids = []
        for node in nodes:
            for key in ['cluster_id', 'created_at', 'data', 'domain',
                        'id', 'index', 'init_at', 'metadata', 'name',
                        'physical_id', 'profile_id', 'profile_name',
                        'project', 'role', 'status', 'status_reason',
                        'updated_at', 'user']:
                self.assertIn(key, node)
            node_ids.append(node['id'])

        self.assertIn(self.node_id, node_ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy
import mock

from senlin.common import scaleutils as su
from senlin.engine import cluster as cm
from senlin.objects import node as no
from senlin.policies import batch_policy as bp
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


class TestBatchPolicy(base.SenlinTestCase):

    def setUp(self):
        super(TestBatchPolicy, self).setUp()
        self.context = utils.dummy_context()
        self.spec = {
            'type': 'senlin.policy.batch',
            'version': '1.0',
            'properties': {
                'min_in_service': 1,
                'max_batch_size': 2,
                'pause_time': 60,
            }
        }

    def test_policy_init(self):
        policy = bp.BatchPolicy('test-batch', self.spec)

        self.assertIsNone(policy.id)
        self.assertEqual('test-batch', policy.name)
        self.assertEqual('senlin.policy.batch-1.0', policy.type)
        self.assertEqual(1, policy.min_in_service)
        self.assertEqual(2, policy.max_batch_size)
        self.assertEqual(60, policy.pause_time)

    def test__cal_batch_size_for_update(self):
        policy = bp.BatchPolicy('test-batch', self.spec)

        size, number = policy._cal_batch_size(5, 'CLUSTER_UPDATE')
        self.assertEqual(2, size)
        self.assertEqual(3, number)

    def test__cal_batch_size_for_delete_greater_than_max(self):
        policy = bp.BatchPolicy('test-batch', self.spec)

        size, number = policy._cal_batch_size(5, 'CLUSTER_DELETE')
        self.assertEqual(2, size)
        self.assertEqual(3, number)

    def test__cal_batch_size_for_delete_less_than_max(self):
        spec = copy.deepcopy(self.spec)
        spec['properties']['max_batch_size'] = 3

        policy = bp.BatchPolicy('test-batch', spec)

        size, number = policy._cal_batch_size(4, 'CLUSTER_DELETE')
        self.assertEqual(2, size)
        self.assertEqual(2, number)

    def test__cal_batch_size_less_than_min(self):
        spec = copy.deepcopy(self.spec)
        spec['properties']['min_in_service'] = 2
        policy = bp.BatchPolicy('test-batch', spec)

        size, number = policy._cal_batch_size(1, 'CLUSTER_UPDATE')
        self.assertEqual(1, size)
        self.assertEqual(1, number)

    def test__cal_batch_size_with_default_max(self):
        spec = copy.deepcopy(self.spec)
        spec['properties']['max_batch_size'] = -1

        policy = bp.BatchPolicy('test-batch', spec)

        size, number = policy._cal_batch_size(5, 'CLUSTER_UPDATE')
        self.assertEqual(4, size)
        self.assertEqual(2, number)

    def test__pick_nodes_all_active(self):
        node1 = mock.Mock(id='1', status='ACTIVE')
        node2 = mock.Mock(id='2', status='ACTIVE')
        node3 = mock.Mock(id='3', status='ACTIVE')

        bad = []
        good = [node1, node2, node3]

        policy = bp.BatchPolicy('test-batch', self.spec)
        nodes = policy._pick_nodes(2, 2, bad, good)

        self.assertEqual(2, len(nodes))
        self.assertIn(node1.id, nodes[0])
        self.assertIn(node2.id, nodes[0])
        self.assertIn(node3.id, nodes[1])

    def test__pick_nodes_with_error_nodes(self):
        node1 = mock.Mock(id='1', status='ACTIVE')
        node2 = mock.Mock(id='2', status='ACTIVE')
        node3 = mock.Mock(id='3', status='ERROR')

        bad = [node3.id]
        good = [node1, node2]

        policy = bp.BatchPolicy('test-batch', self.spec)

        nodes = policy._pick_nodes(2, 2, bad, good)

        self.assertEqual(2, len(nodes))
        self.assertIn(node3.id, nodes[0])
        self.assertIn(node1.id, nodes[0])
        self.assertIn(node2.id, nodes[1])

    @mock.patch.object(bp.BatchPolicy, '_pick_nodes')
    @mock.patch.object(bp.BatchPolicy, '_cal_batch_size')
    @mock.patch.object(su, 'filter_error_nodes')
    @mock.patch.object(no.Node, 'count_by_cluster')
    def test__create_plan_for_update(self, mock_count, mock_filter,
                                     mock_cal, mock_pick):
        action = mock.Mock()
        action.context = self.context
        action.action = 'CLUSTER_UPDATE'
        cluster = mock.Mock(id='cid')
        node1 = mock.Mock(id='1', status='ACTIVE')
        node2 = mock.Mock(id='2', status='ACTIVE')
        node3 = mock.Mock(id='3', status='ACTIVE')
        cluster.nodes = [node1, node2, node3]

        count = 3
        mock_count.return_value = count
        mock_filter.return_value = ([], [node1, node2, node3])
        mock_cal.return_value = (2, 2)
        mock_pick.return_value = [{'1', '2'}, {'3'}]

        policy = bp.BatchPolicy('test-batch', self.spec)
        res, plan = policy._create_plan(cluster, action)
        excepted_plan = {
            'pause_time': self.spec['properties']['pause_time'],
            'plan': [{'1', '2'}, {'3'}]
        }
        self.assertTrue(res)
        self.assertEqual(excepted_plan, plan)

        mock_count.assert_called_once_with(action.context, cluster.id)
        mock_filter.assert_called_once_with(cluster.nodes)
        mock_cal.assert_called_once_with(count, 'CLUSTER_UPDATE')
        mock_pick.assert_called_once_with(2, 2, [], [node1, node2, node3])

    @mock.patch.object(bp.BatchPolicy, '_cal_batch_size')
    @mock.patch.object(no.Node, 'count_by_cluster')
    def test__create_plan_for_delete(self, mock_count, mock_cal):
        action = mock.Mock()
        action.context = self.context
        action.action = 'CLUSTER_DELETE'
        cluster = mock.Mock(id='cid')
        node1 = mock.Mock(id='1', status='ACTIVE')
        node2 = mock.Mock(id='2', status='ACTIVE')
        node3 = mock.Mock(id='3', status='ACTIVE')
        cluster.nodes = [node1, node2, node3]

        count = 3
        mock_count.return_value = count
        mock_cal.return_value = (2, 2)

        policy = bp.BatchPolicy('test-batch', self.spec)
        res, plan = policy._create_plan(cluster, action)
        excepted_plan = {
            'pause_time': self.spec['properties']['pause_time'],
            'batch_size': 2
        }
        self.assertTrue(res)
        self.assertEqual(excepted_plan, plan)

        mock_count.assert_called_once_with(action.context, cluster.id)
        mock_cal.assert_called_once_with(count, 'CLUSTER_DELETE')

    @mock.patch.object(no.Node, 'count_by_cluster')
    def test__create_plan_for_update_no_node(self, mock_count):
        action = mock.Mock()
        action.context = self.context
        action.action = 'CLUSTER_UPDATE'
        cluster = mock.Mock(id='cid')
        mock_count.return_value = 0

        policy = bp.BatchPolicy('test-batch', self.spec)
        excepted_plan = {
            'pause_time': self.spec['properties']['pause_time'],
            'plan': [{}]
        }
        res, value = policy._create_plan(cluster, action)
        self.assertTrue(res)
        self.assertEqual(excepted_plan, value)
        mock_count.assert_called_once_with(action.context, cluster.id)

    @mock.patch.object(no.Node, 'count_by_cluster')
    def test__create_plan_for_delete_no_node(self, mock_count):
        action = mock.Mock()
        action.context = self.context
        action.action = 'CLUSTER_DELETE'
        cluster = mock.Mock(id='cid')
        mock_count.return_value = 0

        policy = bp.BatchPolicy('test-batch', self.spec)
        excepted_plan = {
            'pause_time': self.spec['properties']['pause_time'],
            'batch_size': 0,
        }
        res, value = policy._create_plan(cluster, action)
        self.assertTrue(res)
        self.assertEqual(excepted_plan, value)
        mock_count.assert_called_once_with(action.context, cluster.id)

    @mock.patch.object(bp.BatchPolicy, '_create_plan')
    @mock.patch.object(cm.Cluster, 'load')
    def test_pre_op_for_update(self, mock_load, mock_plan):
        action = mock.Mock()
        action.context = self.context
        action.action = 'CLUSTER_UPDATE'
        cluster = mock.Mock(id='cid')

        mock_load.return_value = cluster

        excepted_plan = {
            'pause_time': self.spec['properties']['pause_time'],
            'plan': [{'1', '2'}, {'3'}]
        }
        mock_plan.return_value = (True, excepted_plan)

        policy = bp.BatchPolicy('test-batch', self.spec)
        policy.pre_op(cluster.id, action)

        mock_load.assert_called_once_with(action.context, cluster.id)
        mock_plan.assert_called_once_with(cluster, action)

    @mock.patch.object(bp.BatchPolicy, '_create_plan')
    @mock.patch.object(cm.Cluster, 'load')
    def test_pre_op_for_delete(self, mock_load, mock_plan):
        action = mock.Mock()
        action.context = self.context
        action.action = 'CLUSTER_DELETE'
        cluster = mock.Mock(id='cid')

        mock_load.return_value = cluster

        excepted_plan = {
            'pause_time': self.spec['properties']['pause_time'],
            'batch_size': 2,
        }
        mock_plan.return_value = (True, excepted_plan)

        policy = bp.BatchPolicy('test-batch', self.spec)
        policy.pre_op(cluster.id, action)

        mock_load.assert_called_once_with(action.context, cluster.id)
        mock_plan.assert_called_once_with(cluster, action)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys

import openstackdocstheme

from senlin.version import version_info as senlin_version

sys.path.insert(0, os.path.abspath('../..'))
# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.todo',
    'sphinx.ext.graphviz',
    'sphinx.ext.intersphinx',
    'oslosphinx',
    'oslo_config.sphinxext',
]

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'senlin'
copyright = u'2015, OpenStack Foundation'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents
#
# "version" and "release" are used by the "log-a-bug" feature
#
# The short X.Y version.
# The full version, including alpha/beta/rc tags.
release = senlin_version.version_string_with_vcs()
# The short X.Y version.
version = senlin_version.canonical_version_string()

# A few variables have to be set for the log-a-bug feature.
#   giturl: The location of conf.py on Git. Must be set manually.
#   gitsha: The SHA checksum of the bug description. Extracted from git log.
#   bug_tag: Tag for categorizing the bug. Must be set manually.
#   bug_project: Launchpad project to file bugs against.
# These variables are passed to the logabug code via html_context.
giturl = u'https://git.openstack.org/cgit/openstack/senlin/tree/doc/source'
git_cmd = ["git", "log", "--pretty=format:'%ad, commit %h", "--date=local",
           "-n1"]
gitsha = subprocess.Popen(git_cmd,
                          stdout=subprocess.PIPE).communicate()[0]
bug_tag = "docs"

# source tree
pwd = os.getcwd()

# html_context allows us to pass arbitrary values into the html template
html_context = {"pwd": pwd,
                "gitsha": gitsha,
                "bug_tag": bug_tag,
                "giturl": giturl,
                "bug_project": "senlin"}


# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# -- Options for HTML output --------------------------------------------------

# html_static_path = ['static']

# The theme to use for HTML and HTML Help pages. See the documentation for a
# list of builtin themes.
html_theme = 'openstackdocs'

# Add any paths that contain custom themes here, relative to this directory
html_theme_path = [openstackdocstheme.get_html_theme_path()]

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index',
     '%s.tex' % project,
     u'%s Documentation' % project,
     u'OpenStack Foundation', 'manual'),
]

# Example configuration for intersphinx: refer to the Python standard library.
# intersphinx_mapping = {'http://docs.python.org/': None}

[extensions]
# todo_include_todos = True
